@@ -3,11 +3,15 @@ from datetime import datetime
 from enum import Enum
 from typing import Any, ClassVar, Dict, List, Literal, Optional, TypedDict, Union
 
+JSONType = Union[Dict[str, Any], List[Any], str, int, float, bool, None]
+
 from openai.types import CompletionUsage
 from openai.types.chat.chat_completion_message import (
-    ChatCompletionMessageToolCall,
     FunctionCall,
 )
+from openai.types.chat.chat_completion_message_tool_call import (
+    ChatCompletionMessageToolCall,
+)
 from pydantic import BaseModel, ConfigDict, Field
 
 from eval_protocol.get_pep440_version import get_pep440_version
@@ -110,6 +114,7 @@ class Status(BaseModel):
         # Custom codes for EP (using higher numbers to avoid conflicts)
         FINISHED = 100
         RUNNING = 101
+        SCORE_INVALID = 102
 
     @classmethod
     def rollout_running(cls) -> "Status":
@@ -163,6 +168,13 @@ class Status(BaseModel):
         """Create a status indicating the rollout failed with an error."""
         return cls(code=cls.Code.INTERNAL, message=error_message, details=details or [])
 
+    @classmethod
+    def score_invalid(
+        cls, message: str = "Score is invalid", details: Optional[List[Dict[str, Any]]] = None
+    ) -> "Status":
+        """Create a status indicating the score is invalid."""
+        return cls(code=cls.Code.SCORE_INVALID, message=message, details=details or [])
+
     def is_running(self) -> bool:
         """Check if the status indicates the rollout is running."""
         return self.code == self.Code.RUNNING
@@ -179,6 +191,10 @@ class Status(BaseModel):
         """Check if the status indicates the rollout was stopped."""
         return self.code == self.Code.CANCELLED
 
+    def is_score_invalid(self) -> bool:
+        """Check if the status indicates the score is invalid."""
+        return self.code == self.Code.SCORE_INVALID
+
     def get_termination_reason(self) -> Optional[TerminationReason]:
         """Extract termination reason from details if present."""
         for detail in self.details:
@@ -222,6 +238,33 @@ class ChatCompletionContentPartTextParam(BaseModel):
     text: str = Field(..., description="The text content.")
     type: Literal["text"] = Field("text", description="The type of the content part.")
 
+    # Provide dict-like access for tests and ergonomic usage
+    def __getitem__(self, key: str) -> Any:
+        if key == "text":
+            return self.text
+        if key == "type":
+            return self.type
+        raise KeyError(key)
+
+    def get(self, key: str, default: Any = None) -> Any:
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def keys(self):
+        return (k for k in ("text", "type"))
+
+    def values(self):
+        return (self.text, self.type)
+
+    def items(self):
+        return [("text", self.text), ("type", self.type)]
+
+    def __iter__(self):
+        # Iterate over keys only
+        return iter(["text", "type"])
+
 
 class Message(BaseModel):
     """Chat message model with trajectory evaluation support."""
@@ -230,16 +273,35 @@ class Message(BaseModel):
     content: Optional[Union[str, List[ChatCompletionContentPartTextParam]]] = Field(
         default="", description="The content of the message."
     )
+    reasoning_content: Optional[str] = Field(
+        default=None, description="Optional hidden chain-of-thought or reasoning content."
+    )
     name: Optional[str] = None
     tool_call_id: Optional[str] = None
     tool_calls: Optional[List[ChatCompletionMessageToolCall]] = None
     function_call: Optional[FunctionCall] = None
     control_plane_step: Optional[Dict[str, Any]] = None
+    weight: Optional[int] = None
+
+    def dump_mdoel_for_chat_completion_request(self):
+        """Only keep chat completion accepted fields"""
+        return self.model_dump(exclude_none=True, exclude={"control_plane_step", "reasoning_content", "weight"})
 
     @classmethod
     def model_validate(cls, obj, *args, **kwargs):
-        if isinstance(obj, dict) and "role" not in obj:
-            raise ValueError("Role is required")
+        if isinstance(obj, dict):
+            if "role" not in obj:
+                raise ValueError("Role is required")
+            # Be lenient: if tool_calls entries are missing required 'id', synthesize one
+            tool_calls_obj = obj.get("tool_calls")
+            if isinstance(tool_calls_obj, list):
+                fixed_tool_calls = []
+                for tc in tool_calls_obj:
+                    if isinstance(tc, dict):
+                        if not tc.get("id"):
+                            tc = {**tc, "id": generate_id()}
+                    fixed_tool_calls.append(tc)
+                obj = {**obj, "tool_calls": fixed_tool_calls}
         return super().model_validate(obj, *args, **kwargs)
 
 
@@ -255,6 +317,7 @@ class MetricResult(BaseModel):
     is_score_valid: bool = True
     score: float = Field(..., ge=0.0, le=1.0)
     reason: str
+    data: Dict[str, Any] = Field(default_factory=dict, description="Optional extra metric data for debugging.")
 
     def __getitem__(self, key: str) -> Any:
         if key in self.__fields__:  # Changed to __fields__ for Pydantic v1 compatibility
@@ -276,10 +339,12 @@ class MetricResult(BaseModel):
         return [getattr(self, key) for key in self.__fields__.keys()]  # Changed to __fields__
 
     def items(self):
-        return [(key, getattr(self, key)) for key in self.__fields__.keys()]  # Changed to __fields__
+        # Exclude 'data' from items to keep items hashable and match tests
+        return [(key, getattr(self, key)) for key in self.__fields__.keys() if key != "data"]  # Changed to __fields__
 
     def __iter__(self):
-        return iter(self.__fields__.keys())  # Changed to __fields__
+        # Exclude 'data' to match expectations in tests
+        return iter([k for k in self.__fields__.keys() if k != "data"])  # Changed to __fields__
 
 
 class StepOutput(BaseModel):
@@ -397,15 +462,11 @@ class EvaluateResult(BaseModel):
 
 CompletionParams = Dict[str, Any]
 """
-Common set of completion parameters that most model providers support in their
-API. Set total=False to allow extra fields since LiteLLM + providers have their
-own set of parameters. The following parameters are common fields that are
-populated.
-
-model: str
-temperature: Optional[float]
-max_tokens: Optional[int]
-top_p: Optional[float]
+The completion parameters for the respective LLM SDK or agent framework.
+Depending on the rollout processor, this might be the parameters passed to
+LiteLLM completion call or parameters for the "run" method of the "Agent" class
+in Pydantic AI.  You can also customize this dictionary to whatever you need if
+you implement your own custom rollout processor.
 """
 
 
@@ -426,10 +487,10 @@ class InputMetadata(BaseModel):
         default_factory=dict, description="Completion endpoint parameters used"
     )
     dataset_info: Optional[Dict[str, Any]] = Field(
-        None, description="Dataset row details: seed, system_prompt, environment_context, etc"
+        default=None, description="Dataset row details: seed, system_prompt, environment_context, etc"
     )
     session_data: Optional[Dict[str, Any]] = Field(
-        None, description="Session metadata like timestamp (input only, no duration/usage)"
+        default=None, description="Session metadata like timestamp (input only, no duration/usage)"
     )
 
 
@@ -443,11 +504,19 @@ class EvaluationThreshold(BaseModel):
     success: float = Field(
         ..., description="Minimum success rate threshold (fraction of total score, 0.0 to 1.0)", ge=0.0, le=1.0
     )
-    standard_error: Optional[float] = Field(
-        None, description="Maximum standard error threshold (fraction of total score, 0.0 to 1.0)", ge=0.0, le=1.0
+    standard_error: float | None = Field(
+        default=None,
+        description="Maximum standard error threshold (fraction of total score, 0.0 to 1.0)",
+        ge=0.0,
+        le=1.0,
     )
 
 
+class EvaluationThresholdDict(TypedDict):
+    success: float
+    standard_error: float | None
+
+
 class EvalMetadata(BaseModel):
     """Metadata about the evaluation that was run."""
 
@@ -466,6 +535,16 @@ class EvalMetadata(BaseModel):
     passed: Optional[bool] = Field(None, description="Whether the evaluation passed based on the threshold")
 
 
+class CostMetrics(BaseModel):
+    """Cost metrics for LLM API calls."""
+
+    input_cost: Optional[float] = Field(None, description="Cost in USD for input tokens.")
+
+    output_cost: Optional[float] = Field(None, description="Cost in USD for output tokens.")
+
+    total_cost_dollar: Optional[float] = Field(None, description="Total cost in USD for the API call.")
+
+
 class ExecutionMetadata(BaseModel):
     """Metadata about the execution of the evaluation."""
 
@@ -485,10 +564,26 @@ class ExecutionMetadata(BaseModel):
     )
 
     run_id: Optional[str] = Field(
-        None,
+        default=None,
         description=("The ID of the run that this row belongs to."),
     )
 
+    usage: Optional[CompletionUsage] = Field(
+        default=None, description="Token usage statistics from LLM calls during execution."
+    )
+
+    cost_metrics: Optional[CostMetrics] = Field(default=None, description="Cost breakdown for LLM API calls.")
+
+    duration_seconds: Optional[float] = Field(
+        default=None,
+        description="Processing duration in seconds for this evaluation row. Note that if it gets retried, this will be the duration of the last attempt.",
+    )
+
+    experiment_duration_seconds: Optional[float] = Field(
+        default=None,
+        description="Processing duration in seconds for an entire experiment. Note that includes time it took for retries.",
+    )
+
 
 class EvaluationRow(BaseModel):
     """
@@ -500,6 +595,8 @@ class EvaluationRow(BaseModel):
     supporting both row-wise batch evaluation and trajectory-based RL evaluation.
     """
 
+    model_config = ConfigDict(extra="allow")
+
     # Core OpenAI ChatCompletion compatible conversation data
     messages: List[Message] = Field(description="List of messages in the conversation. Also known as a trajectory.")
 
@@ -510,7 +607,7 @@ class EvaluationRow(BaseModel):
 
     # Input-related metadata (grouped together for cleaner organization)
     input_metadata: InputMetadata = Field(
-        default_factory=InputMetadata,
+        default_factory=lambda: InputMetadata(),
         description="Metadata related to the input (dataset info, model config, session data, etc.).",
     )
 
@@ -520,8 +617,8 @@ class EvaluationRow(BaseModel):
     )
 
     # Ground truth reference (moved from EvaluateResult to top level)
-    ground_truth: Optional[str] = Field(
-        default=None, description="Optional ground truth reference for this evaluation."
+    ground_truth: Optional[JSONType] = Field(
+        default=None, description="JSON-serializable ground truth reference for this evaluation."
     )
 
     # Unified evaluation result
@@ -530,15 +627,10 @@ class EvaluationRow(BaseModel):
     )
 
     execution_metadata: ExecutionMetadata = Field(
-        default_factory=ExecutionMetadata,
+        default_factory=lambda: ExecutionMetadata(run_id=None),
         description="Metadata about the execution of the evaluation.",
     )
 
-    # LLM usage statistics
-    usage: Optional[CompletionUsage] = Field(
-        default=None, description="Token usage statistics from LLM calls during execution."
-    )
-
     created_at: datetime = Field(default_factory=datetime.now, description="The timestamp when the row was created.")
 
     eval_metadata: Optional[EvalMetadata] = Field(
@@ -576,6 +668,20 @@ class EvaluationRow(BaseModel):
         """Returns only the assistant messages from the conversation."""
         return [msg for msg in self.messages if msg.role == "assistant"]
 
+    def last_assistant_message(self) -> Optional[Message]:
+        """Returns the last assistant message from the conversation. Returns None if none found."""
+        assistant_messages = self.get_assistant_messages()
+        if not assistant_messages:
+            return None
+        return assistant_messages[-1]
+
+    def get_first_user_message(self) -> Optional[Message]:
+        """Returns the first user message from the conversation. Returns None if none found."""
+        user_messages = self.get_user_messages()
+        if not user_messages:
+            return None
+        return user_messages[0]
+
     def get_user_messages(self) -> List[Message]:
         """Returns only the user messages from the conversation."""
         return [msg for msg in self.messages if msg.role == "user"]
@@ -593,27 +699,35 @@ class EvaluationRow(BaseModel):
     def get_total_reward(self) -> float:
         """Get total reward from control_plane_step data."""
         messages_with_control_plane = [msg for msg in self.messages if msg.control_plane_step]
-        return (
-            sum(msg.control_plane_step["reward"] for msg in messages_with_control_plane)
-            if messages_with_control_plane
-            else 0.0
-        )
+        if not messages_with_control_plane:
+            return 0.0
+        total = 0.0
+        for msg in messages_with_control_plane:
+            step = msg.control_plane_step or {}
+            try:
+                total += float(step.get("reward", 0.0))
+            except (TypeError, ValueError):
+                continue
+        return total
 
     def get_terminated(self) -> bool:
         """Get termination status from control_plane_step data."""
         messages_with_control_plane = [msg for msg in self.messages if msg.control_plane_step]
-        return (
-            any(msg.control_plane_step["terminated"] for msg in messages_with_control_plane)
-            if messages_with_control_plane
-            else False
-        )
+        if not messages_with_control_plane:
+            return False
+        for msg in messages_with_control_plane:
+            step = msg.control_plane_step or {}
+            if bool(step.get("terminated", False)):
+                return True
+        return False
 
     def get_termination_reason(self) -> str:
         """Get termination reason from the final control_plane_step data."""
         # Find the last message with control_plane_step that has termination_reason
         for msg in reversed(self.messages):
             if msg.control_plane_step and msg.control_plane_step.get("termination_reason"):
-                return msg.control_plane_step["termination_reason"]
+                reason = msg.control_plane_step.get("termination_reason")
+                return str(reason)
         return "unknown"
 
     def __hash__(self) -> int:
